# Copyright (C) 2019  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""Unit tests for the message schema."""

import copy
import unittest

from jsonschema import ValidationError
from .. import schema


class BuildChrootStartedV1Test(unittest.TestCase):
    """A set of unit tests to ensure the schema works as expected."""

    msg_class = schema.BuildChrootStartedV1
    required_fields = ['status', 'what', 'chroot', 'ip', 'user', 'who',
                       'pid', 'copr', 'version', 'build', 'owner', 'pkg']

    def setUp(self):
        self.fedmsg_message = {
            "username": "copr",
            "source_name": "datanommer",
            "certificate": "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0tCk1JSUVUakNDQTdlZ0F3SUJBZ0lDQVBZd0RRWUpL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\nUlRJRklDQVRFLS0tLS0K\n",
            "i": 1,
            "timestamp": 1561545908.0,
            "msg_id": "2019-77336a2c-bbe1-4362-b466-32d6595d328b",
            "crypto": "x509",
            "topic": "org.fedoraproject.prod.copr.build.start",
            "headers": {},
            "signature": "kMANxtkdgupoXP5VAbyp6Hc//G+i4KGUhaRvlStb1uSNMRoUG+rqeNio4JXwez9C9ppT7oGezTAn\nSNfW2F0mt/XikqQrhpP3cOpE47327udgP+IwrP+WT3lZv1wH9LTpHj+eRgKmXNcCNUXYCWvCgGqi\nQ5PSpDcZsn+MJaCwP/M=\n",
            "source_version": "0.9.0",
            "msg": {
                "status": 3,
                "what": "build start: user:churchyard copr:python3.8 pkg:python-hypothesis build:945012 ip:172.25.93.20 pid:3762",
                "chroot": "fedora-rawhide-x86_64",
                "ip": "172.25.93.20",
                "user": "churchyard",
                "who": "backend.worker-23551-PC",
                "pid": 3762,
                "copr": "python3.8",
                "version": "4.23.8-1.fc31",
                "build": 945012,
                "owner": "@python",
                "pkg": "python-hypothesis"
            }
        }
        self.build_id = 945012
        self.name = "@python/python3.8"

    def test_correct_messages(self):
        """
        Assert the message schema validates a correct message.
        """
        message = self.msg_class(body=self.fedmsg_message)
        message.validate()

    def test_missing_fields(self):
        """Assert an exception is actually raised on validation failure."""
        for key in self.required_fields:
            body = copy.deepcopy(self.fedmsg_message)
            del body['msg'][key]
            message = self.msg_class(body=body)
            self.assertRaises(ValidationError, message.validate)

    def test_bad_type(self):
        int_fields = ['status', 'build', 'pid']
        for key in self.required_fields:
            body = copy.deepcopy(self.fedmsg_message)

            print(key)
            if key in int_fields:
                body['msg'][key] = "str"
            else:
                body['msg'][key] = 1

            message = self.msg_class(body=body)
            self.assertRaises(ValidationError, message.validate)

    def test_str(self):
        message = self.msg_class(body=self.fedmsg_message)
        assert message.project_full_name == self.name
        assert message.build_id == self.build_id

    def test_no_wrap(self):
        message = self.msg_class(body=self.fedmsg_message['msg'])
        message.validate()

    def test_nevr(self):
        message = self.msg_class(body=self.fedmsg_message['msg'])
        assert message.package_name == 'python-hypothesis'
        assert message.package_version == '4.23.8'
        assert message.package_release == '1.fc31'
        assert message.package_epoch is None


class BuildChrootEndedV1Test(BuildChrootStartedV1Test):
    msg_class = schema.BuildChrootEndedV1

    def setUp(self):
        self.fedmsg_message = {
            "username": "copr",
            "source_name": "datanommer",
            "certificate": "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0tCk1JSUVUakNDQTdlZ0F3SUJBZ0lDQVBZd0RRWUpL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\nUlRJRklDQVRFLS0tLS0K\n",
            "i": 3,
            "timestamp": 1561043053.0,
            "msg_id": "2019-f916a75e-6a89-48e2-a4e5-53e71fef92fa",
            "crypto": "x509",
            "topic": "org.fedoraproject.prod.copr.build.end",
            "headers": {},
            "signature": "GWWEIfcVvLh59XWrYmDlJoRpxJ3DKYFBwAojVwWncbvfj3gVwJ33UHP5GJXTCsk+gvEUQdn07/Fs\nw9hGIfuiXlH7EjqOFvaiHIScq0VxLvwsManjbij9Ud8au9rLs9NkyS/pdLKq5QtwgXO9Y9LnsZB5\nyWURshXlLZsbZfq7fsE=\n",
            "source_version": "0.9.0",
            "msg": {
                "status": 0,
                "what": "build end: user:kbaig copr:test build:942697 pkg:tkn version:0.1.2-1.fc30 ip:172.25.91.143 pid:26197 status:0",
                "chroot": "fedora-29-x86_64",
                "ip": "172.25.91.143",
                "user": "kbaig",
                "who": "backend.worker-14783-PC",
                "pid": 26197,
                "copr": "test",
                "version": "0.1.2-1.fc30",
                "build": 942697,
                "owner": "kbaig",
                "pkg": "tkn"
            }
        }
        self.build_id = 942697
        self.name = "kbaig/test"

    def test_nevr(self):
        message = self.msg_class(body=self.fedmsg_message['msg'])
        assert message.package_name == 'tkn'
        assert message.package_version == '0.1.2'
        assert message.package_release == '1.fc30'
        assert message.package_epoch is None


class BuildChrootStartedV1DontUseTest(BuildChrootStartedV1Test):
    msg_class = schema.BuildChrootStartedV1DontUse

    def setUp(self):
        self.fedmsg_message = {
            "username": "copr",
            "source_name": "datanommer",
            "certificate": "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0tCk1JSUVUakNDQTdlZ0F3SUJBZ0lDQVBZd0RRWUpL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\nUlRJRklDQVRFLS0tLS0K\n",
            "i": 2,
            "timestamp": 1561550552.0,
            "msg_id": "2019-99461b95-66c1-4097-b4ec-ab2e3ba59047",
            "crypto": "x509",
            "topic": "org.fedoraproject.prod.copr.chroot.start",
            "headers": {},
            "signature": "S4N4QRS7IPAfrng3wwQWxQjjnCUcSQtAvbwZu4IPwVQs1uUv+gbSITIDblCVPCudPZ3yWL4ApP/L\nfNJepkUp/+mu0vDpRZQ+ZzOFSSIlemSaO8Ce1k8uymwbFhrU1L+7lMkY1Q+C4JCj/kXmn8Gs7hWF\noyrmQx0vAraBJc+ZIzQ=\n",
            "source_version": "0.9.0",
            "msg": {
                "status": 3,
                "what": "chroot start: chroot:fedora-29-aarch64 user:praiskup copr:ping pkg:dummy-pkg build:945056 ip:38.145.48.104 pid:9272",
                "chroot": "fedora-29-aarch64",
                "ip": "38.145.48.104",
                "user": "praiskup",
                "who": "backend.worker-23731-AARCH64",
                "pid": 9272,
                "copr": "ping",
                "version": "10:20190626_1356-0",
                "build": 945056,
                "owner": "praiskup",
                "pkg": "dummy-pkg"
            }
        }
        self.build_id = 945056
        self.name = "praiskup/ping"

    def test_nevr(self):
        message = self.msg_class(body=self.fedmsg_message['msg'])
        assert message.package_name == "dummy-pkg"
        assert message.package_version == "20190626_1356"
        assert message.package_release == "0"
        assert message.package_epoch == '10'


class BuildChrootStompOldStartTest(unittest.TestCase):
    msg_class = schema.BuildChrootStartedV1Stomp

    required = [
        "build",
        "owner",
        "copr",
        "submitter",
        "package",
        "chroot",
        "builder",
        "status",
        "status_int",
    ]

    int_fields = [
        'build',
        'status_int'
    ]

    def setUp(self):
        self.fedmsg_message = {
            "build": "38492",
            "owner": "praiskup",
            "copr": "ping",
            "submitter": "None",
            "package": "None-None",
            "chroot":
            "srpm-builds",
            "builder": "10.8.29.188",
            "status": "SUCCEEDED",
            "status_int": "1"
        }
        self.build_id = 38492
        self.name = "praiskup/ping"

    def test_correct_messages(self):
        """
        Assert the message schema validates a correct message.
        """
        message = self.msg_class(body=self.fedmsg_message)
        message.validate()

    def test_package(self):
        message = self.msg_class(body=self.fedmsg_message)
        message.validate()
        assert message.status == "succeeded"

    def test_nevr(self):
        message = self.msg_class(body=self.fedmsg_message)
        assert message.package_name == None
        assert message.package_version == None
        assert message.package_release == None
        assert message.package_epoch is None


class BuildChrootStompOldEndTest(BuildChrootStompOldStartTest):
    msg_class = schema.BuildChrootEndedV1Stomp

    def setUp(self):
        self.fedmsg_message = {
            "build": "38492",
            "owner": "praiskup",
            "copr": "ping",
            "submitter": "praiskup",
            "package": "dummy-pkg-1:20190701_2127-0",
            "chroot": "fedora-rawhide-x86_64",
            "builder": "10.0.150.161",
            "status": "SUCCEEDED",
            "status_int": "1"
        }
        self.build_id = 38492
        self.name = "praiskup/ping"

    def test_nevr(self):
        message = self.msg_class(body=self.fedmsg_message)
        assert message.package_name == 'dummy-pkg'
        assert message.package_version == '20190701_2127'
        assert message.package_release == '0'
        assert message.package_epoch == '1'


class BuildChrootStartedV1StompDontUseTest(unittest.TestCase):
    msg_class = schema.BuildChrootStartedV1StompDontUse

    def setUp(self):
        self.fedmsg_message = {
            "chroot": "fedora-29-x86_64"
        }

    def test_chroot(self):
        msg = self.msg_class(body=self.fedmsg_message)
        msg.validate()



# Copyright (C) 2019  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
Private hierarchy of Copr messages.  This is for module-internal purposes only.
"""

from fedora_messaging import message

class _CoprMessage(message.Message):
    def __init__(self, *args, **kwargs):
        if 'body' in kwargs:
            body = kwargs.pop('body')
            if 'msg' in body:
                body = body['msg']
            kwargs['body'] = body

        super(_CoprMessage, self).__init__(*args, **kwargs)

    """
    Base class that all Copr messages should inherit from.
    """
    def __str__(self):
        return "Unspecified Copr message"

    def _str_prefix(self):
        return "Copr Message"


class _CoprProjectMessage(_CoprMessage):
    def _str_prefix(self):
        return '{0} in project "{1}"'.format(
            super(_CoprProjectMessage, self)._str_prefix(),
            self.project_full_name,
        )

    @property
    def project_owner(self):
        """
        Owner name of the Copr project.  It may be either ``@groupname`` or
        ``username`` (leading `at` sign indicates group).
        """
        raise NotImplementedError

    @property
    def project_name(self):
        """
        Name of the copr project.  Note that several owners may host project of
        the same name at the same time.
        """
        raise NotImplementedError

    @property
    def project_full_name(self):
        """
        Project owner name + project name, separated by slash.
        """
        return '{0}/{1}'.format(self.project_owner, self.project_name)


class _BuildMessage(_CoprProjectMessage):
    def _str_prefix(self):
        return '{0}: build {1}'.format(
            super(_BuildMessage, self)._str_prefix(),
            self.build_id,
        )

    @property
    def build_id(self):
        """
        Copr Build ID.

        Note that one copr build (identified by this ID) generates several build
        messages (namely for each :py:attr:`.chroot`, before build started and
        after the build finished).
        """
        raise NotImplementedError

    @property
    def package_name(self):
        """
        *Name* of the *package* this message is related to
        [#footnote_may_be_unknown]_.
        """
        raise NotImplementedError

    # TODO: from old messages, it's not possible to detect package's
    # architecture (yes, chroot ... but that doesn't resolve noarch packages).

    def _evr(self):
        # return (epoch, version, release) triplet
        raise NotImplementedError

    @property
    def package_version(self):
        """
        *Version* of the *package* [#footnote_may_be_unknown]_.  Returns `None`
        if epoch is unset.
        """
        _, version, _ = self._evr()
        return version

    @property
    def package_release(self):
        """
        *Release* of the *package* [#footnote_may_be_unknown]_.  Returns `None`
        if epoch is unset.
        """
        _, _, release = self._evr()
        return release

    @property
    def package_epoch(self):
        """
        *Epoch* of the *package* [#footnote_may_be_unknown]_.  Returns `None` if
        epoch is unset.
        """
        epoch, _, _ = self._evr()
        return epoch

    @property
    def package_full_name(self):
        """
        Full - human readable - package name (not meant to be parsed).
        [#footnote_may_be_unknown]_.
        """
        if not self.package_name:
            return None

        (epoch, version, release) = self._evr()

        return "{name}-{epoch}{version}-{release}".format(
            name=self.package_name,
            epoch=epoch + ":" if epoch else "",
            version=version,
            release=release,
        )


class _BuildChrootMessage(_BuildMessage):
    @property
    def chroot(self):
        """
        Build chroot name this build is done in.  For example
        ``fedora-rawhide-x86_64`` or ``epel-7-x86_64``.

        When this is source build, the returned value is ``srpm-builds``.  Each
        build (see :py:attr:`.build_id`) in copr needs to prepare sources first
        (those are imported into copr dist-git) for the following binary-RPM
        builds; so such source build might represent some SCM method execution,
        or e.g. just act of downloading SRPM from a remote URL.
        """
        raise NotImplementedError



# Copyright (C) 2019  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import stomp
import json

from .schema import (
        BuildChrootStartedV1,
        BuildChrootEndedV1,
        BuildChrootStartedV1DontUse,
        BuildChrootStartedV1Stomp,
        BuildChrootEndedV1Stomp,
        BuildChrootStartedV1StompDontUse,
)

from .private.consumer import _GenericConsumer


def message_object_from_raw(headers, body):
    destination = headers.get('destination')
    topic = headers.get('topic')
    body = json.loads(body)

    if destination.endswith('.copr.build.end'):
        return BuildChrootEndedV1(body=body)
    if destination.endswith('.copr.build.start'):
        return BuildChrootStartedV1(body=body)
    if destination.endswith('.copr.chroot.start'):
        return BuildChrootStartedV1DontUse(body=body)

    if topic == 'build.start':
        return BuildChrootStartedV1Stomp(body=body)
    if topic == 'build.end':
        return BuildChrootEndedV1Stomp(body=body)
    if topic == 'chroot.start':
        return BuildChrootStartedV1StompDontUse(body=body)


class Consumer(stomp.listener.ConnectionListener, _GenericConsumer):

    def on_message(self, headers, body):
        message = message_object_from_raw(headers, body)
        message.validate()

        if isinstance(message, BuildChrootStartedV1StompDontUse):
            return

        try:
            if isinstance(message, BuildChrootStartedV1) or \
               isinstance(message, BuildChrootStartedV1Stomp):
                self.build_chroot_started(message)
                return

            if isinstance(message, BuildChrootEndedV1) or \
               isinstance(message, BuildChrootEndedV1Stomp):
                self.build_chroot_ended(message)
                return

        except NotImplementedError:
            return



# Copyright (C) 2019  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from fedora_messaging.message import Message

from .private.consumer import _GenericConsumer
from .schema import (
        BuildChrootEnded,
        BuildChrootEndedV1,
        BuildChrootStarted,
        BuildChrootStartedV1,
        BuildChrootStartedV1DontUse,
)


def retype_message(message):
    if type(message) == Message:
        # This message might be actually originated from fedmsg, and the
        # fedmsg->AMQP proxy made things wrong and the fedora-messaging
        # tooling is unable to correctly instanitate it.
        if message.topic.endswith('.copr.build.end'):
            return BuildChrootEndedV1(body=message.body)
        if message.topic.endswith('.copr.build.start'):
            return BuildChrootStartedV1(body=message.body)
        if message.topic.endswith('.copr.chroot.start'):
            return BuildChrootStartedV1DontUse(body=message.body)

    return message


class Consumer(_GenericConsumer):
    """
    Helper.
    """
    def __call__(self, message):
        message = retype_message(message)

        message.validate()

        if isinstance(message, BuildChrootStartedV1DontUse):
            return

        try:
            if isinstance(message, BuildChrootStarted):
                self.build_chroot_started(message)
                return

            if isinstance(message, BuildChrootEnded):
                self.build_chroot_ended(message)
                return
        except NotImplementedError:
            pass


class Printer(Consumer):
    def build_chroot_started(self, message):
        print("build_chroot_started(): {}".format(message))

    def build_chroot_ended(self, message):
        print("build_chroot_ended(): {}".format(message))



#! /usr/bin/python3
#
# Copyright (C) 2019  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, "README.md")) as fd:
    README = fd.read()

__name__ = 'copr-messaging'
__description__ = "A schema and tooling for Copr fedora-messaging"
__author__ = "Copr team"
__author_email__ = "copr-devel@lists.fedorahosted.org"
__url__ = "https://pagure.io/copr/copr"


with open(os.path.join(here, "requirements.txt")) as f:
    __requires__ = f.read().split("\n")[:-1]

setup(
    name=__name__,
    version="0.4",
    description=__description__,
    long_description=README,
    url=__url__,

    # Possible options are at https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
        "Programming Language :: Python :: 3.7",
    ],
    license="GPLv2+",
    maintainer="Copr Team",
    maintainer_email=__author_email__,
    keywords="fedora",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=__requires__,
    test_suite="copr_messaging.tests",
    entry_points={
        "fedora.messages": [
            "copr.build.start=copr_messaging.schema:BuildChrootStartedV1",
            "copr.build.end=copr_messaging.schema:BuildChrootEndedV1",

            # TODO: drop those entry points;  these shouldn't be needed once
            # all message consumers moved to `copr_messaging` module.
            "copr.chroot.start=copr_messaging.schema:BuildChrootStartedV1DontUse",
            "copr.unused.build.start=copr_messaging.schema:BuildChrootStartedV1Stomp",
            "copr.unused.build.end=copr_messaging.schema:BuildChrootEndedV1Stomp",
            "copr.unused.chroot.start=copr_messaging.schema:BuildChrootStartedV1StompDontUse",
        ]
    },
)



# Copyright (C) 2019  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


from copr_common.enums import StatusEnum
from .hierarchy import _BuildChrootMessage

class _PreFMBuildMessage(_BuildChrootMessage):
    """ old obsoleted msg format """

    """
    This is 'copr.build.end' message schema.

    This message is from the pre-fedora-messaging era.
    """
    body_schema = {
        "id": "http://fedoraproject.org/message-schema/copr#",
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description":
            "Message sent by Copr build system",
        "type": "object",
        "required": [
            "status",
            "chroot",
            "build",
            "owner",
            "copr",
            "pkg",
            "version",
            "what",
            "ip",
            "who",
            "user",
            "pid",
        ],
        "properties": {
            "status": {
                "type": "number",
                "description": "numerical representation of build status",
            },
            "chroot": {
                "type": "string",
                "description": "what chroot was this build run against, "
                               "'srpm-builds' for source builds",
            },
            "owner": {
                "type": "string",
                "description":
                    "owner (grup/user) of the project this build was done in",
            },
            "copr": {
                "type": "string",
                "description": "name of the project the build was built in",
            },
            "build": {
                "type": "number",
                "description": "build id",
            },
            "pkg": {
                "type": ['string', 'null'],
                "description": "Package name, null if unknown",
            },
            "version": {
                "type": ['string', 'null'],
                "description": "Package version, null if unknown",
            },
            "user": {
                "type": ['string', 'null'],
                "description": "Copr user who submitted the build, null if unknown",
            },
            "what": {
                "type": "string",
                "description": "combination of all the fields",
            },
            "ip": {
                "type": ["string", "null"],
                "description": "IP address (usually not public) of the builder",
            },
            "who": {
                "type": ['string'],
                "description": "what python module has sent this message",
            },
            "pid": {
                "type": 'number',
                "description": "process ID of the process on backend taking "
                               "care of the task",
            },
        },
    }

    @property
    def build_id(self):
        return self.body['build']

    @property
    def project_name(self):
        return str(self.body['copr'])

    @property
    def project_owner(self):
        return str(self.body['owner'])

    @property
    def chroot(self):
        return self.body['chroot']

    @property
    def status(self):
        """
        String representation of build chroot status.
        """
        return StatusEnum(self.body['status'])

    @property
    def package_name(self):
        return self.body.get('pkg')

    def _evr(self):
        evr = self.body.get('version')
        if not evr:
            return (None, None, None)

        e_v = evr .split(':', 1)
        epoch = None
        if len(e_v) == 1:
            v_r = e_v[0]
        else:
            epoch = e_v[0]
            v_r = e_v[1]

        version, release = v_r.split('-', 1)
        return (epoch, version, release)



# Copyright (C) 2019  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


"""
TODO
"""

import copy

from copr_common.enums import StatusEnum

from .hierarchy import _BuildChrootMessage


BODY_SCHEMA = {
    "id": "http://fedoraproject.org/message-schema/copr#",
    "$schema": "http://json-schema.org/draft-04/schema#",
    "type": "object",
    "required": [
        "build",
        "owner",
        "copr",
        "submitter",
        "package",
        "chroot",
        "builder",
        "status",
        "status_int",
    ],
    "properties": {
        "build": {
            "type": "string",
            "description": "build id as string",
        },
        "owner": {
            "type": "string",
            "description":
                "owner (grup/user) of the project this build was done in",
        },
        "copr": {
            "type": "string",
            "description": "name of the project the build was built in",
        },
        "submitter": {
            "type": ["string", "null"],
            "description": "who (copr user) submitted this build",
        },
        "package": {
            "type": ['string', 'null'],
            "description": "Package NVR, null if unknown",
        },
        "chroot": {
            "type": "string",
            "description": "what chroot was this build run against, "
                           "'srpm-builds' for source builds",
        },
        "builder": {
            "type": ["string", "null"],
            "description": "IP address (usually not public) of the builder",
        },
        "status": {
            "type": "string",
            "description": "text representation of build status",
        },
        "status_int": {
            "type": "string",
            "description": "integer representation of build status",
        },
    },
}


class _OldStompChrootMessage(_BuildChrootMessage):
    body_schema = copy.deepcopy(BODY_SCHEMA)
    body_schema.update({
        'description': 'hey!',
    })

    @property
    def build_id(self):
        return int(self.body['build'])

    @property
    def project_name(self):
        return str(self.body['copr'])

    @property
    def project_owner(self):
        return str(self.body['owner'])

    @property
    def chroot(self):
        return self.body['chroot']

    @property
    def status(self):
        """
        String representation of build chroot status.
        """
        return StatusEnum(int(self.body['status_int']))

    def _nevr(self):
        nevr = self.body.get('package')
        if nevr == 'None-None':
            # compat hack, for mistakenly sent strings like this before
            return (None, None, None, None)

        nev, release = nevr.rsplit('-', 1)
        name, e_v = nev.rsplit('-', 1)
        e_v_list = e_v.split(':')
        if len(e_v_list) == 1:
            epoch = None
            version = e_v_list[0]
        else:
            epoch = e_v_list[0]
            version = e_v_list[1]

        return (name, epoch, version, release)

    def _evr(self):
        _, epoch, version, release = self._nevr()
        return (epoch, version, release)

    @property
    def package_name(self):
        name, _, _, _ = self._nevr()
        return name



# Copyright (C) 2019  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
This file contains schemas for messages sent by Copr project.
"""

import copy

from copr_common.enums import StatusEnum
from fedora_messaging import message

from .private.hierarchy import _BuildChrootMessage
from .private.schema_old import _PreFMBuildMessage
from .private import schema_stomp_old


class BuildChrootEnded(_BuildChrootMessage):
    """
    Representation of a message sent by Copr build system right after some Copr
    worker finished a build in a particular mock chroot.
    """
    @property
    def status(self):
        """
        string representation of build status, e.g. ``succeeded``, ``failed``
        """
        raise NotImplementedError

    def __str__(self):
        return '{0}: chroot "{1}" ended as "{2}".'.format(
            super(BuildChrootEnded, self)._str_prefix(),
            self.chroot,
            self.status,
        )


class BuildChrootStarted(_BuildChrootMessage):
    """
    Representation of a message sent by Copr build system right before some Copr
    worker starts working on a build in a particular mock chroot.
    """
    def __str__(self):
        return '{0}: chroot "{1}" started.'.format(
            super(BuildChrootStarted, self)._str_prefix(),
            self.chroot,
        )


class BuildChrootStartedV1(_PreFMBuildMessage, BuildChrootStarted):
    """
    schema for the old fedmsg-era 'copr.build.start' message
    """
    topic = 'copr.build.start'


class BuildChrootEndedV1(_PreFMBuildMessage, BuildChrootEnded):
    """
    schema for the old fedmsg-era 'copr.build.end' message
    """
    topic = 'copr.build.end'

    @property
    def status(self):
        return StatusEnum(self.body['status'])


class BuildChrootStartedV1DontUse(_PreFMBuildMessage, BuildChrootStarted):
    """
    Schema for the old fedmsg-era 'copr.chroot.start' message, this message
    duplicated the 'copr.build.start' message, so you should never use this.
    """
    topic = 'copr.chroot.start'


class BuildChrootStartedV1Stomp(schema_stomp_old._OldStompChrootMessage,
                                BuildChrootStarted):
    topic = 'build.start'
    body_schema = copy.deepcopy(schema_stomp_old.BODY_SCHEMA)
    body_schema.update({
        'description': ""
    })


class BuildChrootStartedV1StompDontUse(message.Message):
    topic = 'chroot.start'

    body_schema = {
        "id": "http://fedoraproject.org/message-schema/copr#",
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description":
            "Message sent by Copr build system when build in "
            "concrete chroot started",
        "type": "object",
        "required": [
            "chroot",
        ],
        "properties": {
            "chroot": {
                "type": "string",
                "description": "what chroot was this build run against, "
                               "'srpm-builds' for source builds",
            },
        },
    }


class BuildChrootEndedV1Stomp(schema_stomp_old._OldStompChrootMessage,
                              BuildChrootEnded):
    topic = 'build.end'

    body_schema = copy.deepcopy(schema_stomp_old.BODY_SCHEMA)
    body_schema.update({
        'description': ""
    })

    @property
    def status(self):
        return StatusEnum(int(self.body['status_int']))
